@@ -1,11 +1,9 @@
 # Copyright (C) 2008 John Paulett (john -at- paulett.org)
-# Copyright (C) 2009-2021 David Aguilar (davvid -at- gmail.com)
+# Copyright (C) 2009-2024 David Aguilar (davvid -at- gmail.com)
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
-from __future__ import absolute_import, division, unicode_literals
-
 import collections
 import os
 import unittest
@@ -51,6 +49,9 @@ class Capture:
         self.args = args
         self.kwargs = kwargs
 
+    def __repr__(self):
+        return object.__repr__(self) + ('(%r, %r)' % (self.args, self.kwargs))
+
 
 class ThingWithProps:
     def _get_identity(self):
@@ -164,6 +165,15 @@ class MyPropertiesDict:
         )
 
 
+# see issue #478
+class SafeData:
+    __slots__ = ()
+
+
+class SafeString(str, SafeData):
+    __slots__ = ()
+
+
 def on_missing_callback(class_name):
     # not actually a runtime problem but it doesn't matter
     warnings.warn("The unpickler couldn't find %s" % class_name, RuntimeWarning)
@@ -194,13 +204,13 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual({tags.B64: encoded}, self.pickler.flatten(data))
 
     def test_decode_base85(self):
-        pickled = {tags.B85: 'P{Y4;Xv4O{u^=-c'}
-        expected = u'P\u00ffth\u00f6\u00f1 3!'.encode('utf-8')
+        expected = 'Pÿthöñ 3!'.encode('utf-8')
+        pickled = {tags.B85: util.b85encode(expected)}
         self.assertEqual(expected, self.unpickler.restore(pickled))
 
     def test_base85_still_handles_base64(self):
-        pickled = {tags.B64: 'UMO/dGjDtsOxIDMh'}
-        expected = u'P\u00ffth\u00f6\u00f1 3!'.encode('utf-8')
+        expected = 'Pÿthöñ 3!'.encode('utf-8')
+        pickled = {tags.B64: util.b64encode(expected)}
         self.assertEqual(expected, self.unpickler.restore(pickled))
 
     def test_string(self):
@@ -391,14 +401,12 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual(inflated.themodule, None)
 
     def test_thing_with_submodule(self):
-        from distutils import sysconfig
-
         obj = Thing('with-submodule')
-        obj.submodule = sysconfig
+        obj.submodule = collections
 
         flattened = self.pickler.flatten(obj)
         inflated = self.unpickler.restore(flattened)
-        self.assertEqual(inflated.submodule, sysconfig)
+        self.assertEqual(inflated.submodule, collections)
 
     def test_type_reference(self):
         """This test ensures that users can store references to types."""
@@ -498,42 +506,47 @@ class PicklingTestCase(unittest.TestCase):
         self.assertTrue(cls is int)
 
     def test_unpickler_on_missing(self):
-        class SimpleClass(object):
-            def __init__(self, i):
-                self.i = i
+        encoded = jsonpickle.encode(Outer.Middle.Inner())
+        self.assertTrue(isinstance(jsonpickle.decode(encoded), Outer.Middle.Inner))
 
-        frozen = jsonpickle.encode(SimpleClass(4))
-        del SimpleClass
+        # Alter the encoded string to create cases where the class is missing
+        # at multiple levels.
+        self.assertTrue(
+            encoded == '{"py/object": "jsonpickle_test.Outer.Middle.Inner"}'
+        )
+        missing_cases = [
+            '{"py/object": "MISSING.Outer.Middle.Inner"}',
+            '{"py/object": "jsonpickle_test.MISSING.Middle.Inner"}',
+            '{"py/object": "jsonpickle_test.Outer.MISSING.Inner"}',
+            '{"py/object": "jsonpickle_test.Outer.Middle.MISSING"}',
+        ]
 
-        # https://docs.python.org/3/library/warnings.html#testing-warnings
+        for case in missing_cases:
+            # https://docs.python.org/3/library/warnings.html#testing-warnings
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter("always")
+                jsonpickle.decode(case, on_missing='warn')
+                self.assertTrue(issubclass(w[-1].category, UserWarning))
+                self.assertTrue(
+                    "Unpickler._restore_object could not find" in str(w[-1].message)
+                )
+
+                jsonpickle.decode(case, on_missing=on_missing_callback)
+                self.assertTrue(issubclass(w[-1].category, RuntimeWarning))
+                self.assertTrue("The unpickler couldn't find" in str(w[-1].message))
 
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            jsonpickle.decode(frozen, on_missing='warn')
-            self.assertTrue(issubclass(w[-1].category, UserWarning))
             self.assertTrue(
-                "Unpickler._restore_object could not find" in str(w[-1].message)
+                jsonpickle.decode(case, on_missing='ignore')
+                == jsonpickle.backend.json.loads(case)
             )
 
-            jsonpickle.decode(frozen, on_missing=on_missing_callback)
-            self.assertTrue(issubclass(w[-1].category, RuntimeWarning))
-            self.assertTrue("The unpickler couldn't find" in str(w[-1].message))
-
-        self.assertTrue(
-            jsonpickle.decode(frozen, on_missing='ignore')
-            == {
-                'py/object': 'jsonpickle_test.PicklingTestCase.test_unpickler_on_missing.<locals>.SimpleClass',
-                'i': 4,
-            }
-        )
-
-        try:
-            jsonpickle.decode(frozen, on_missing='error')
-        except jsonpickle.errors.ClassNotFoundError:
-            # it's supposed to error
-            self.assertTrue(True)
-        else:
-            self.assertTrue(False)
+            try:
+                jsonpickle.decode(case, on_missing='error')
+            except jsonpickle.errors.ClassNotFoundError:
+                # it's supposed to error
+                self.assertTrue(True)
+            else:
+                self.assertTrue(False)
 
     def test_private_slot_members(self):
         obj = jsonpickle.loads(jsonpickle.dumps(MySlots()))
@@ -982,11 +995,22 @@ class JSONPickleTestCase(SkippableTest):
         self.assertTrue(True)
 
     def test_depth_tracking(self):
-        l = []
+        liszt = []
         pickler = jsonpickle.Pickler()
-        pickler.flatten([l, l, l, l, l])
+        pickler.flatten([liszt, liszt, liszt, liszt, liszt])
         self.assertEqual(pickler._depth, -1)
 
+    def test_readonly_attrs(self):
+        s = SafeString("test")
+        pickled = jsonpickle.encode(s, handle_readonly=True)
+        pickled_json_dict = jsonpickle.backend.json.loads(pickled)
+        # make sure it's giving concise output by erroring if it includes
+        # a default method which is unnecessary
+        self.assertTrue("join" not in pickled_json_dict)
+        unpickled = jsonpickle.decode(pickled)
+        self.assertEqual(unpickled.__class__, SafeString)
+        self.assertEqual(unpickled, s)
+
 
 class PicklableNamedTuple(object):
     """
