@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
 import datetime
 import warnings
 
@@ -15,7 +14,6 @@ except ImportError:
 import jsonpickle
 import jsonpickle.ext.numpy
 from jsonpickle import handlers
-from jsonpickle.compat import PY2
 
 
 @pytest.fixture(scope='module', autouse=True)
@@ -128,15 +126,14 @@ def test_ndarray_roundtrip():
         np.array([0, 1, -1, np.inf, -np.inf, np.nan], dtype='f2'),
     ]
 
-    if not PY2:
-        arrays.extend(
-            [
-                np.rec.array(
-                    [('NGC1001', 11), ('NGC1002', 1.0), ('NGC1003', 1.0)],
-                    dtype=[('target', 'S20'), ('V_mag', 'f4')],
-                )
-            ]
-        )
+    arrays.extend(
+        [
+            np.rec.array(
+                [('NGC1001', 11), ('NGC1002', 1.0), ('NGC1003', 1.0)],
+                dtype=[('target', 'S20'), ('V_mag', 'f4')],
+            )
+        ]
+    )
     for array in arrays:
         decoded = roundtrip(array)
         assert_equal(decoded, array)
@@ -199,10 +196,7 @@ def test_weird_arrays():
     a.strides = 1
 
     # this is kinda fishy; a has overlapping memory, _a does not
-    if PY2:
-        warn_count = 0
-    else:
-        warn_count = 1
+    warn_count = 1
     with warnings.catch_warnings(record=True) as w:
         _a = roundtrip(a)
         assert len(w) == warn_count
@@ -253,10 +247,7 @@ def test_transpose():
     b = a[1:, 1:]
     assert b.base is a
 
-    if PY2:
-        warn_count = 0
-    else:
-        warn_count = 1
+    warn_count = 1
     with warnings.catch_warnings(record=True) as w:
         _a, _b = roundtrip([a, b])
         assert len(w) == warn_count
@@ -275,10 +266,7 @@ def test_buffer():
     """test behavior with memoryviews which are not ndarrays"""
     bstring = 'abcdefgh'.encode('utf-8')
     a = np.frombuffer(bstring, dtype=np.byte)
-    if PY2:
-        warn_count = 0
-    else:
-        warn_count = 1
+    warn_count = 1
     with warnings.catch_warnings(record=True) as w:
         _a = roundtrip(a)
         npt.assert_array_equal(a, _a)
@@ -292,10 +280,7 @@ def test_as_strided():
     is not an ndarray.
 
     """
-    if PY2:
-        warn_count = 0
-    else:
-        warn_count = 1
+    warn_count = 1
     a = np.arange(10)
     b = np.lib.stride_tricks.as_strided(a, shape=(5,), strides=(a.dtype.itemsize * 2,))
     data = [a, b]
@@ -339,8 +324,8 @@ def test_byteorder():
 
 
 def test_zero_dimensional_array():
-    expect = np.array(0.0)
-    actual = roundtrip(expect)
+    expect = np.array(float(0.0), dtype="float64")
+    actual = jsonpickle.decode(jsonpickle.encode(expect))
     npt.assert_array_equal(expect, actual)
 
 
@@ -385,5 +370,11 @@ def test_np_random():
         assert expect == actual
 
 
+def test_np_poly1d():
+    # issue 391, test poly1d roundtrip
+    obj = np.poly1d([1, 2, 3])
+    assert obj == jsonpickle.decode(jsonpickle.encode(obj))
+
+
 if __name__ == '__main__':
     pytest.main([__file__])
