@@ -2,21 +2,27 @@
 
 import unittest
 
-import jsonpickle
-
 from helper import SkippableTest
 
+import jsonpickle
+
 try:
     import sqlalchemy as sqa
+    from sqlalchemy import orm
     from sqlalchemy.ext import declarative
     from sqlalchemy.orm import Session
+
     HAS_SQA = True
 except ImportError:
     HAS_SQA = False
 
 if HAS_SQA:
-
-    Base = declarative.declarative_base()
+    # sqlalchemy.ext.declarative.declarative_base() was deprecated in SQLAlchemy 2.0
+    # and replaced by sqlalchemy.orm.declarative_base().
+    if hasattr(orm, 'declarative_base'):
+        Base = orm.declarative_base()
+    else:
+        Base = declarative.declarative_base()
 
     class Table(Base):
         __tablename__ = 'table'
@@ -26,7 +32,6 @@ if HAS_SQA:
 
 
 class SQLAlchemyTestCase(SkippableTest):
-
     def setUp(self):
         """Create a new sqlalchemy engine for the test"""
 
@@ -95,9 +100,11 @@ class SQLAlchemyTestCase(SkippableTest):
 
         meta = sqa.MetaData()
         expect = sqa.Table(
-            'test', meta,
+            'test',
+            meta,
             sqa.Column('id', sqa.Integer()),
-            sqa.Column('text', sqa.Text()))
+            sqa.Column('text', sqa.Text()),
+        )
 
         jsonstr = jsonpickle.dumps(expect)
         actual = jsonpickle.loads(jsonstr)
@@ -107,15 +114,12 @@ class SQLAlchemyTestCase(SkippableTest):
         # These must be unique instances
         self.assertNotEqual(expect.metadata, actual.metadata)
         # Columns names must exactly match
-        self.assertEqual(sorted(expect.columns.keys()),
-                         sorted(actual.columns.keys()))
+        self.assertEqual(sorted(expect.columns.keys()), sorted(actual.columns.keys()))
         # As should the types
         self.assertEqual(expect.c.id.name, actual.c.id.name)
-        self.assertEqual(expect.c.id.type.__class__,
-                         actual.c.id.type.__class__)
+        self.assertEqual(expect.c.id.type.__class__, actual.c.id.type.__class__)
         self.assertEqual(expect.c.text.name, actual.c.text.name)
-        self.assertEqual(expect.c.text.type.__class__,
-                         actual.c.text.type.__class__)
+        self.assertEqual(expect.c.text.type.__class__, actual.c.text.type.__class__)
 
 
 def suite():
